/*
 * Sample_SSAO.h
 *
 * @author Simon
 */

#include "SdkSample.h"

#ifndef SAMPLE_SSAO_H_
#define SAMPLE_SSAO_H_

using namespace Ogre;
using namespace OgreBites;

class _OgreSampleClassExport Sample_SSAO : public SdkSample
{
private:
	SceneNode* mKnot;

	std::vector<String> mMeshNames;
	std::vector<Entity*> mMeshes;
	int mCurrentMeshIndex;

	std::vector<String> mCompositorNames;
	String mCurrentCompositor;
	
	std::vector<String> mPostNames;
	String mCurrentPost;

	std::vector<Widget *> mCreasePanel;
	std::vector<Widget *> mUnsharpPanel;
	std::vector<Widget *> mBilateralPanel;
	
	CheckBox* mSamplingCheckbox;
	Slider* mScreenSpaceSlider;
	Slider* mWorldSpaceSlider;

	Slider* mSampleLengthExponent;

	Slider* mAngleBiasSlider;
	Slider* mOffsetStepSlider;
	Slider* mEdgeHighlight;
	Slider* mDefaultAccessibility;

	SelectMenu* mCameraMenu;

	bool mTakeScreenshot;

	/**
	 * show all the widgets in the panel
	 * @see Sample_SSAO::showWidget
	 */
	void showPanel(std::vector<Widget *> panel);

	/**
	 * hide all the widgets in the panel
	 * @see Sample_SSAO::hideWidget
	 */
	void hidePanel(std::vector<Widget *> panel);

	/**
	 * Show the given widget. This sets it visible and re-adds it to the panel
	 */
	void showWidget(Widget* widget);

	/**
	 * Hide the given widget. This sets it to not visible and also removes it
	 * from the panel.
	 */
	void hideWidget(Widget* widget);

public:
	Sample_SSAO();
	StringVector getRequiredPlugins();
	void testCapabilities(const RenderSystemCapabilities* caps);

protected:
	/**
	 * Setup the compositors to be used.
	 */
	void setupCompositors();

	/**
	 * Setup the controls, ie. the gui elements.
	 */
	void setupControls();

	/**
	 * create the scene and load the content.
	 * This function is called from the sample browser when the sample is
	 * started. It is kind of the main entry point.
	 */
	void setupContent();

	/**
	 * Change the current displayed mesh to the new mesh identified by its index.
	 * @param index The index of the new mesh in the mesh <code>vector</code>.
	 */
	void changeMesh(int index);

	/**
	 * Change the compositor to be used.
	 * @param compositor the name of the compositor
	 */
	void changeCompositor(String compositor);

	/**
	 * change the post filter to be used.
	 * @param post the name of the new post filter.
	 */
	void changePost(String post);

	// sdkTray listener callbacks
	void itemSelected(SelectMenu* menu);
	void sliderMoved(Slider* slider);
	void buttonHit(OgreBites::Button* button);
	void checkBoxToggled(CheckBox *box);

	/**
	 * The frame rendering queued listener. It is used to take screenshots
	 * without gui elements.
	 */
	bool frameRenderingQueued(const FrameEvent & evt);


	// the following three methods are taken/stolen from the ShaderSystem sample.
	/** @see Sample::mousePressed. */
	bool mousePressed(const OIS::MouseEvent& evt, OIS::MouseButtonID id);

	/** @see Sample::mouseReleased. */
	bool mouseReleased(const OIS::MouseEvent& evt, OIS::MouseButtonID id);

	/** @see Sample::mouseMoved. */
	bool mouseMoved(const OIS::MouseEvent& evt);

	/**
	 * Set the uniform value in the compositor
	 * @param compositor The name of the compositor
	 * @param material The material that contains the uniform
	 * @param uniform the name of the uniform parameter
	 * @param value the value
	 * @param setVisible whether to set the compositor to visible or not.
	 * @param position the position at which the compositor should be added again.
	 * defaults to -1, which means that the compositor is readded at the end of the chain.
	 */
	void setUniform(String compositor, String material, String uniform, float value, bool setVisible = true, int position = -1);
};



#endif /* SAMPLE_SSAO_H_ */
